 //**********************************************************************
 // ECAN dsPIC33EP
 //----------------------------------------------------------------------
 #include "built_in.h"
 #include "ECAN_Defs.h"
 //---------------------------------------------------------------------
 unsigned int Can_Init_Flags, Can_Send_Flags, Can_Rcv_Flags;  // can flags
unsigned int RxDataLen;     // received data length in bytes
unsigned int TxDataLen;
char RxData[8];            // can rx/tx data buffer
char TxData[8];
char Msg_Rcvd;             // reception flag
unsigned long RxID;
unsigned long TxID;
//-----------------------------------------------------------------------
const unsigned long VREF = 3300;   // Voltage reference in mV
unsigned char TimerInterruptCount;
unsigned int TemperatureRaw ;
 //----------------------------------------------------------------------
 void mcu_Init(void);
 void CAN_Init(void);
 void CanWrite(void);
 void Delay_ms (unsigned int delay);
 //---------------------------------------------------------------------
 void Timer1_Interrupt() iv IVT_ADDR_T1INTERRUPT ics ICS_AUTO 
{
  TimerInterruptCount ++;    // Increment interrupt counter
  IFS0bits.T1IF = 0;         // Clear Interrupt Flag
}
 
 //---------------------------------------------------------------------
 // Can1 Interrupt routines
 //---------------------------------------------------------------------
 void CAN1_Interrupt() iv IVT_ADDR_C1INTERRUPT ics ICS_AUTO 
{
  IFS2bits.C1IF = 0;       // clear ECAN interrupt flag
 if(C1INTFbits.TBIF)
  {                        // was it tx interrupt?
    C1INTFbits.TBIF = 0;   // if yes clear tx interrupt flag
     LATD.B5 ^=1;
     //------------------------------------------------------
    // Tx Interrupt is working ,this can be used to Tx CAN
    // messages on Interrupt Qeue the messages and Tx them on
    // interrupt from buffer
    //--------------------------------------------------------
  }
 //--------------------------------------------------------------------
 if(C1INTFbits.RBIF)
 {                        // was it rx interrupt?
   C1INTFbits.RBIF = 0;   // if yes clear rx interrupt flag
     Msg_Rcvd = ECAN1Read(&RxID,RxData,&RxDataLen,&Can_Rcv_Flags);
  if(Msg_Rcvd!=0)
  {
    switch (RxID)
   {
     case 0x200 :
     LATD.B2 ^=1;   // Blink Led RD2
     break;
     case 0x311:
     LATD.B3 ^=1;  // Blink Led RD3
     break;
     case 0x420 :
     LATD.B4 ^=1;  // Blink Led RD4
     break;

   }// end switch case
  }
 }
}
//--------------------------------------------------------------------
//main code
//--------------------------------------------------------------------
void main()
{
/*Configure Oscillator to operate the device at 70Mhz
 Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 Fosc= 8M*70/(2*2)=140Mhz for 8M input clock */

 PLLFBD = 68;              /* M=70 */
 CLKDIVbits.PLLPOST = 0;   /* N1=2 */
 CLKDIVbits.PLLPRE = 0;    /* N2=2 */
 OSCTUN = 0;               /* Tune FRC oscillator, if FRC is used */
//-----------------------------------------------------------------------
//Initialize Micro
//----------------------------------------------------------------------
   mcu_Init();
//----------------------------------------------------------------------
// Initailize CAN module
//----------------------------------------------------------------------
   CAN_Init();
//----------------------------------------------------------------------
// Initialize ADC for Temperature measurement
//----------------------------------------------------------------------
    ADC1_Init();

//-----------------------------------------------------------------------
// main code loop
//-----------------------------------------------------------------------
 while(1)
 {
  LATD.B1 ^=1;  // Blink Led RD1
  TemperatureRaw = ADC1_Get_Sample(0);  // read LM35 Voltage
  CanWrite();   // Write CAN messages
  Delay_ms(50); // Wait for next CAN cycle time
 
 }
}
//--------------------------------------------------------------------
void CanWrite()
{
 TxData[0] = (TemperatureRaw & 0xFF); // send LM35 Voltage over CAN
 TxData[1] = (TemperatureRaw >> 8);
 TxData[2] = 0;
 TxData[3] = 0;
 TxData[4] = 0;
 TxData[5] = 0;
 TxData[6] = 0;
 TxData[7] = 0;
 TxDataLen = 8;
 TxID = 0x210;
 ECAN1Write(TxID,TxData,TxDataLen,Can_Send_Flags);
 //--------------------------------------------------------------------
 TxData[0] = (0);
 TxData[1] = (0);
 TxData[2] = 0;
 TxData[3] = 0;
 TxData[4] = 0;
 TxData[5] = 0;
 TxData[6] = 0;
 TxData[7] = 0;
 TxDataLen = 8;
 TxID = 0x211;
 ECAN1Write(TxID,TxData,TxDataLen,Can_Send_Flags);
}
//-------------------------------------------------------------------------
// Timer Init
//-------------------------------------------------------------------------
void Delay_ms (unsigned int delay)
{
 TimerInterruptCount = 0;  // Clear Interrupt counter flag
 T1CONbits.TCKPS0 = 1;
 T1CONbits.TCKPS1 = 0;     // Prescaler = 01 = 8
 PR1 = 0x22E1;             // (1ms/112nSec) = 8929 = 0x22E1
 IPC0bits.T1IP = 4;        // Set Interrupt Priority lower then ADC
 IEC0bits.T1IE = 1;        // Enable Timer1 interrupts
 T1CONbits.TON = 1;        // Enable Timer1

 while (TimerInterruptCount < delay);//Wait for Interrupt counts to equal delay
 T1CONbits.TON = 0;                  // Disable the Timer
}
//---------------------------------------------------------------------
void mcu_Init()
{
 TRISD = 0;
 LATD = 0;
}
//-------------------------------------------------------------------------
void CAN_Init()
{
 const unsigned int SJW = 1;
 const unsigned int BRP = 14;
 const unsigned int PHSEG1 = 1;
 const unsigned int PHSEG2 = 2;
 const unsigned int PROPSEG = 6;
//------------------------------------------------------------------------
 // all PORT pins Digital
//------------------------------------------------------------------------
  ANSELA = 0x00;
  ANSELB = 0x01;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;
  ANSELG = 0x00;
//-----------------------------------------------------
// Configure IC Pins
//-----------------------------------------------------
  Unlock_IOLOCK();
  PPS_Mapping_NoLock(113,_OUTPUT,_C1TX); //RG1
  PPS_Mapping_NoLock(112,_INPUT,_C1RX);  //RG0
  Lock_IOLOCK();
 /* Clear Interrupt Flags */
  IFS0 = 0;
  IFS1 = 0;
  IFS2 = 0;
  IFS3 = 0;
  IFS4 = 0;

  /* Enable ECAN1 Interrupt */

  IEC2bits.C1IE   = 1;        // enable ECAN1 interrupts
  C1INTEbits.TBIE = 1;        // enable ECAN1 tx interrupt
  C1INTEbits.RBIE = 1;        // enable ECAN1 rx interrupt
  Can_Init_Flags = 0;         //
  Can_Send_Flags = 0;         // clear flags
  Can_Rcv_Flags  = 0;         //

  Can_Send_Flags = _ECAN_TX_PRIORITY_0 &
                   _ECAN_TX_STD_FRAME &
                   _ECAN_TX_NO_RTR_FRAME;

  Can_Init_Flags = _ECAN_CONFIG_SAMPLE_THRICE &
                   _ECAN_CONFIG_PHSEG2_PRG_ON &
                   _ECAN_CONFIG_STD_MSG &
                   _ECAN_CONFIG_MATCH_MSG_TYPE &
                   _ECAN_CONFIG_LINE_FILTER_OFF;


  ECAN1DmaChannelInit(0,1,&ECAN1RxTxRAMBuffer);  // init dma channel 0 for
                                                 // dma to ECAN peripheral transfer
  ECAN1DmaChannelInit(2,0,&ECAN1RxTxRAMBuffer);  // init dma channel 2 for
                                                 // ECAN peripheral to dma transfer

  //Baudrate 250Kb/sec @70MIPS
  ECAN1Initialize(SJW,BRP,PHSEG1,PHSEG2,PROPSEG,Can_Init_Flags);
  ECAN1SetBufferSize(ECAN1RAMBUFFERSIZE);          // set number of rx+tx buffers in DMA RAM

  ECAN1SelectTxBuffers(0x000F);                    // select transmit buffers
                                                   // 0x000F = buffers 0:3 are transmit buffers
  ECAN1SetOperationMode(_ECAN_MODE_CONFIG,0xFF);   // set CONFIGURATION mode

  ECAN1SetMask(_ECAN_MASK_0, -1, _ECAN_CONFIG_MATCH_MSG_TYPE & _ECAN_CONFIG_STD_MSG);  // set all mask1 bits to ones
  ECAN1SetMask(_ECAN_MASK_1, -1, _ECAN_CONFIG_MATCH_MSG_TYPE & _ECAN_CONFIG_STD_MSG);  // set all mask2 bits to ones
  ECAN1SetMask(_ECAN_MASK_2, -1, _ECAN_CONFIG_MATCH_MSG_TYPE & _ECAN_CONFIG_STD_MSG);  // set all mask3 bits to ones
  
  ECAN1SetFilter(_ECAN_FILTER_1,0x200,_ECAN_MASK_1,_ECAN_RX_BUFFER_7,_ECAN_CONFIG_STD_MSG);
  ECAN1SetFilter(_ECAN_FILTER_2,0x311,_ECAN_MASK_1,_ECAN_RX_BUFFER_8,_ECAN_CONFIG_STD_MSG);
  ECAN1SetFilter(_ECAN_FILTER_3,0x420,_ECAN_MASK_1,_ECAN_RX_BUFFER_9,_ECAN_CONFIG_STD_MSG);
  ECAN1SetOperationMode(_ECAN_MODE_NORMAL, 0xFF);  // set NORMAL mode

}
//---------------------------------------------------------------------------